import { jsx } from 'react/jsx-runtime';
import { useState } from 'react';
import { useToast, Progress } from '@chakra-ui/react';
import { UploadStep } from './UploadStep/UploadStep.js';
import { SelectHeaderStep } from './SelectHeaderStep/SelectHeaderStep.js';
import { SelectSheetStep } from './SelectSheetStep/SelectSheetStep.js';
import { mapWorkbook } from '../utils/mapWorkbook.js';
import { ValidationStep } from './ValidationStep/ValidationStep.js';
import { MatchColumnsStep } from './MatchColumnsStep/MatchColumnsStep.js';
import { exceedsMaxRecords } from '../utils/exceedsMaxRecords.js';
import { useRsi } from '../hooks/useRsi.js';

var Type;
(function (Type) {
    Type[Type["upload"] = 0] = "upload";
    Type[Type["selectSheet"] = 1] = "selectSheet";
    Type[Type["selectHeader"] = 2] = "selectHeader";
    Type[Type["matchColumns"] = 3] = "matchColumns";
    Type[Type["validateData"] = 4] = "validateData";
})(Type || (Type = {}));
const UploadFlow = ({ nextStep }) => {
    const [state, setState] = useState({ type: Type.upload });
    const { maxRecords, translations, uploadStepHook, selectHeaderStepHook, matchColumnsStepHook } = useRsi();
    const toast = useToast();
    switch (state.type) {
        case Type.upload:
            return (jsx(UploadStep, { onContinue: async (workbook) => {
                    const isSingleSheet = workbook.SheetNames.length === 1;
                    if (isSingleSheet) {
                        if (maxRecords && exceedsMaxRecords(workbook.Sheets[workbook.SheetNames[0]], maxRecords)) {
                            toast({
                                status: "error",
                                variant: "left-accent",
                                position: "bottom-left",
                                title: `${translations.uploadStep.maxRecordsExceeded(maxRecords.toString())}`,
                                isClosable: true,
                            });
                            return;
                        }
                        const mappedWorkbook = await uploadStepHook(mapWorkbook(workbook));
                        setState({
                            type: Type.selectHeader,
                            data: mappedWorkbook,
                        });
                        nextStep();
                    }
                    else {
                        setState({ type: Type.selectSheet, workbook });
                    }
                } }));
        case Type.selectSheet:
            return (jsx(SelectSheetStep, { sheetNames: state.workbook.SheetNames, onContinue: async (sheetName) => {
                    if (maxRecords && exceedsMaxRecords(state.workbook.Sheets[sheetName], maxRecords)) {
                        toast({
                            status: "error",
                            variant: "left-accent",
                            position: "bottom-left",
                            title: `${translations.uploadStep.maxRecordsExceeded(maxRecords.toString())}`,
                            isClosable: true,
                        });
                        return;
                    }
                    const mappedWorkbook = await uploadStepHook(mapWorkbook(state.workbook, sheetName));
                    setState({
                        type: Type.selectHeader,
                        data: mappedWorkbook,
                    });
                    nextStep();
                } }));
        case Type.selectHeader:
            return (jsx(SelectHeaderStep, { data: state.data, onContinue: async (...args) => {
                    const { data, headerValues } = await selectHeaderStepHook(...args);
                    setState({
                        type: Type.matchColumns,
                        data,
                        headerValues,
                    });
                    nextStep();
                } }));
        case Type.matchColumns:
            return (jsx(MatchColumnsStep, { data: state.data, headerValues: state.headerValues, onContinue: async (values, rawData, columns) => {
                    const data = await matchColumnsStepHook(values, rawData, columns);
                    setState({
                        type: Type.validateData,
                        data,
                    });
                    nextStep();
                } }));
        case Type.validateData:
            return jsx(ValidationStep, { initialData: state.data });
        default:
            return jsx(Progress, { isIndeterminate: true });
    }
};

export { UploadFlow };
